/* $Id: perfctr.h,v 1.7 2001/04/17 00:10:45 mikpe Exp $
 * x86 Performance-Monitoring Counters driver
 *
 * Copyright (C) 1999-2001  Mikael Pettersson
 */
#ifndef _ASM_I386_PERFCTR_H
#define _ASM_I386_PERFCTR_H

struct perfctr_sum_ctrs {
	unsigned long long tsc;
	unsigned long long pmc[18];
};

struct perfctr_low_ctrs {
	unsigned int tsc;
	unsigned int pmc[18];
};

struct perfctr_control_user {
	unsigned int tsc_on;
	unsigned int nrctrs;		/* total # of counters */
	unsigned int nrictrs;		/* last nrictrs are in i-mode */
	unsigned int pmc_map[18];
	unsigned int evntsel[18];	/* one per counter, even on P5 */
	unsigned int evntsel_aux[18];	/* e.g. P4 ESCR contents */
	int ireset[18];			/* <= 0, for i-mode counters */
};

#ifdef __KERNEL__

#if defined(CONFIG_PERFCTR) || defined(CONFIG_PERFCTR_MODULE)

struct perfctr_control {
	struct perfctr_control_user user;
	union {
		unsigned int p5_cesr;
		unsigned int id;
	} k1;
	union {
		unsigned int p4_escr_map[18];
	} k2;
};

/* called from arch/i386/kernel/setup.c:dodgy_tsc() */
extern void perfctr_dodgy_tsc(void);

/* Driver init/exit. */
extern int perfctr_cpu_init(void);
extern void perfctr_cpu_exit(void);

/* CPU type name. */
extern char *perfctr_cpu_name[];

/* Hardware reservation. */
extern const char *perfctr_cpu_reserve(const char *service);
extern void perfctr_cpu_release(const char *service);

/* Check that the user's control data is valid.
   Update the kernel's private control data.
   Returns < 0 if the control data is invalid. */
extern int perfctr_cpu_check_control(struct perfctr_control *control);

/* Update the control and event selection data. PRE: control->nrctrs > 0. */
extern void perfctr_cpu_write_control(const struct perfctr_control *control);

/* Read and save current counter values. PRE: control->nrctrs > 0. */
extern void perfctr_cpu_read_counters(const struct perfctr_control *control,
				      struct perfctr_low_ctrs *ctrs);

/* Process resume/suspend hooks for virtual perfctrs. */
static __inline__ void perfctr_cpu_enable_rdpmc(void) { }
static __inline__ void perfctr_cpu_disable_rdpmc(void) { }

typedef void (*perfctr_ihandler_t)(unsigned long pc);

#ifdef CONFIG_X86_LOCAL_APIC
#include <linux/version.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
#include <asm/fixmap.h>
#include <asm/apic.h>
struct hw_interrupt_type;
#include <asm/hw_irq.h>
#ifdef LOCAL_PERFCTR_VECTOR
#define PERFCTR_INTERRUPT_SUPPORT 1
#endif
#endif
#endif

#if PERFCTR_INTERRUPT_SUPPORT
extern unsigned int apic_lvtpc_irqs[];
extern void perfctr_interrupt(void);
extern void perfctr_cpu_set_ihandler(perfctr_ihandler_t);
extern void perfctr_cpu_isuspend(const struct perfctr_control*);
extern void perfctr_cpu_iresume(const struct perfctr_control*,
				const struct perfctr_low_ctrs*);
#else
static __inline__ void perfctr_cpu_set_ihandler(perfctr_ihandler_t x) { }
static __inline__ void perfctr_cpu_isuspend(const struct perfctr_control *x) { }
static __inline__ void perfctr_cpu_iresume(const struct perfctr_control *x,
					   const struct perfctr_low_ctrs *y) { }
#endif

#else	/* !CONFIG_PERFCTR */

static __inline__ void perfctr_dodgy_tsc(void) { }

#endif	/* CONFIG_PERFCTR */

#endif	/* __KERNEL__ */

#endif	/* _ASM_I386_PERFCTR_H */
